﻿using Kivii.WebDriver;
using Kivii.WebDriver.Actions;
using OpenQA.Selenium.Firefox;
using OpenQA.Selenium.Remote;
using OpenQA.Selenium.Support.Events;
using System;
using System.Windows.Forms;

namespace Kivii.Test.WebDriver
{
    public partial class FrmMain : Form
    {
        private Manager _manager=null;
        public FrmMain()
        {
            InitializeComponent();
        }


        private void appendMsg(string msg)
        {
            rtbMsg.AppendText($"{msg}{Environment.NewLine}");
            rtbMsg.ScrollToCaret();
        }

        #region 事件
        private void addListener()
        {
            if (_manager == null) return;

            _manager.OnException += manager_OnException;
            _manager.OnWindowChanged += manager_OnWindowChanged;

            _manager.ElementClicked += EventDriver_ElementClicked;
            _manager.ElementClicking += EventDriver_ElementClicking;
            _manager.ElementValueChanging += EventDriver_ElementValueChanging;
            _manager.ElementValueChanged += EventDriver_ElementValueChanged;
            _manager.ExceptionThrown += EventDriver_ExceptionThrown;
            _manager.FindElementCompleted += EventDriver_FindElementCompleted;
            _manager.FindingElement += EventDriver_FindingElement;
            _manager.GetShadowRootCompleted += EventDriver_GetShadowRootCompleted;
            _manager.GettingShadowRoot += EventDriver_GettingShadowRoot;
            _manager.Navigated += EventDriver_Navigated;
            _manager.NavigatedBack += EventDriver_NavigatedBack;
            _manager.NavigatedForward += EventDriver_NavigatedForward;
            _manager.Navigating += EventDriver_Navigating;
            _manager.NavigatingBack += EventDriver_NavigatingBack;
            _manager.NavigatingForward += EventDriver_NavigatingForward;
            _manager.ScriptExecuted += EventDriver_ScriptExecuted;
            _manager.ScriptExecuting += EventDriver_ScriptExecuting;
        }

        private void manager_OnWindowChanged(object sender, Window e)
        {
            gbMsg.Text = $"输出(窗口Handle={e.Handle},Title={e.Title})";
        }

        private void manager_OnException(object sender, Exception e)
        {
            appendMsg($"Exception:{DateTime.Now}{Environment.NewLine}{e.Message}{Environment.NewLine}{e.StackTrace}");
        }

        private void EventDriver_ScriptExecuting(object sender, WebDriverScriptEventArgs e)
        {
            appendMsg($"EventDriver_ScriptExecuting:{Environment.NewLine}{e.Script}");
        }

        private void EventDriver_ScriptExecuted(object sender, WebDriverScriptEventArgs e)
        {
            appendMsg($"EventDriver_ScriptExecuted:{Environment.NewLine}{e.Script}");
        }

        private void EventDriver_NavigatingForward(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_NavigatingForward:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_NavigatingBack(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_NavigatingBack:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_Navigating(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_Navigating:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_NavigatedForward(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_NavigatedForward:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_NavigatedBack(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_NavigatedBack:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_Navigated(object sender, WebDriverNavigationEventArgs e)
        {
            appendMsg($"EventDriver_Navigated:{Environment.NewLine}{e.Url}");
        }

        private void EventDriver_GettingShadowRoot(object sender, GetShadowRootEventArgs e)
        {
            appendMsg($"EventDriver_GettingShadowRoot:{Environment.NewLine}{e}");
        }

        private void EventDriver_GetShadowRootCompleted(object sender, GetShadowRootEventArgs e)
        {
            appendMsg($"EventDriver_GetShadowRootCompleted:{Environment.NewLine}{e}");
        }

        private void EventDriver_FindingElement(object sender, FindElementEventArgs e)
        {
            appendMsg($"EventDriver_FindingElement:{Environment.NewLine}{e}");
        }

        private void EventDriver_FindElementCompleted(object sender, FindElementEventArgs e)
        {
            appendMsg($"EventDriver_FindElementCompleted:{Environment.NewLine}{e}");
        }

        private void EventDriver_ExceptionThrown(object sender, WebDriverExceptionEventArgs e)
        {
            appendMsg($"EventDriver_ExceptionThrown:{Environment.NewLine}{e}");
        }

        private void EventDriver_ElementValueChanged(object sender, WebElementValueEventArgs e)
        {
            appendMsg($"EventDriver_ElementValueChanged:{Environment.NewLine}{e}");
        }

        private void EventDriver_ElementValueChanging(object sender, WebElementValueEventArgs e)
        {
            appendMsg($"EventDriver_ElementValueChanging:{Environment.NewLine}{e}");
        }

        private void EventDriver_ElementClicking(object sender, WebElementEventArgs e)
        {
            appendMsg($"EventDriver_ElementClicking:{Environment.NewLine}{e}");
        }

        private void EventDriver_ElementClicked(object sender, WebElementEventArgs e)
        {
            appendMsg($"EventDriver_ElementClicked:{Environment.NewLine}{e}");
        }
        #endregion

        #region 交互事件
        private void btnOpen_Click(object sender, EventArgs e)
        {
            if (_manager != null) return;
            rtbMsg.Clear();
            _manager = new Manager();
            var serviceUrl = string.Empty;
            if (cbxRemote.Checked)
            {
                if (cbUseHub.Checked)
                {
                    serviceUrl = "http://hub.selenium.kivii.org/";
                }
                else
                {
                    serviceUrl = $"http://driver.{cbxBrowserType.Text}.selenium.kivii.org";
                }
            }
            if (_manager.Initialize(cbxBrowserType.Text, serviceUrl))
            {
                addListener();
            }
            else
            {
                appendMsg("初始化失败，可能是未能成功连接WebDriver驱动！");
            }
        }
        private void btnClose_Click(object sender, EventArgs e)
        {
            _manager?.Dispose();
            _manager = null;
        }

        private void btnScreenshot_Click(object sender, EventArgs e)
        {
            //if (_driver == null) return;
            SaveFileDialog sfd=new SaveFileDialog();
            sfd.Filter = "图片文件|*.jpg";
            if (sfd.ShowDialog() != DialogResult.OK) return;
            _manager?.Screenshot(sfd.FileName);
        }

        private void btnLoadScript_Click(object sender, EventArgs e)
        {
            _manager?.LoadScript(tbxScriptPath.Text);
        }

        private void cbxRemote_CheckedChanged(object sender, EventArgs e)
        {
            lnkVncUrl.Enabled = cbxRemote.Checked;
            cbUseHub.Enabled=cbxRemote.Checked;
            cbxBrowserType_SelectedIndexChanged(null, null);
        }
        private void cbxBrowserType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!cbxRemote.Checked)
            {
                lnkVncUrl.Text= null;
                return;
            }
            if (cbUseHub.Checked)
            {
                lnkVncUrl.Text = $"http://hub.selenium.kivii.org";
            }
            else
            {
                if (string.IsNullOrEmpty(cbxBrowserType.Text))
                {
                    lnkVncUrl.Text = null;
                }
                else
                {
                    lnkVncUrl.Text = $"http://viewer.{cbxBrowserType.Text}.selenium.kivii.org";
                }
            }
        }
        private void lnkVncUrl_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            if (cbUseHub.Checked)
            {
                System.Diagnostics.Process.Start($"{lnkVncUrl.Text}");
            }
            else
            {
                System.Diagnostics.Process.Start($"{lnkVncUrl.Text}/?autoconnect=1&resize=scale&password=secret");
            }
        }

        private void btnNavigate_Click(object sender, EventArgs e)
        {
            _manager?.NavigateTo(tbxUrl.Text);
        }
        private void btnExecuteScript_Click(object sender, EventArgs e)
        {
            if (tabScripts.SelectedTab.Tag.ToString() == "engine")//网页内执行
            {
                var result = _manager.Execute(rtbEngineScript.Text);
                rtbEngineScript.Clear();
                appendMsg($"Result:{result}");
            }
            else if (tabScripts.SelectedTab.Tag.ToString() == "web")
            {
                var result = _manager?.ExecuteScript(rtbWebScript.Text);
                rtbWebScript.Clear();
                appendMsg($"Result:{result}");
            }
        }

        private void btnHandlers_Click(object sender, EventArgs e)
        {
            foreach (var handler in _manager.AllWindowHandles)
            {
                appendMsg($"Handler Name:{handler}");
            }
        }

        private void btnClearMsg_Click(object sender, EventArgs e)
        {
            rtbMsg.Clear();
        }

        private void btnAutoAppendScript_Click(object sender, EventArgs e)
        {
            rtbEngineScript.AppendText($"{cbScriptTip.Text}{Environment.NewLine}");
        }



        private void btnTest_Click(object sender, EventArgs e)
        {
            RemoteWebDriver driver = null;
            try
            {
                driver = new RemoteWebDriver(new FirefoxOptions().ToCapabilities());
                //System.Diagnostics.Process.Start($"http://viewer.firefox.selenium.kivii.org/?autoconnect=1&resize=scale&password=secret");
                //driver.Navigate().GoToUrl("https://www.baidu.com");
                driver.ExecuteScript("console.log(1);");
                driver.ExecuteScript("document.getElementByTagName('head');");
                driver.ExecuteScript("alert('1')", "");
            }
            catch (Exception ex)
            {
                appendMsg($"Ex:{Environment.NewLine}{ex.Message}");
            }
            driver.Quit();
        }

        private void btnInjection_Click(object sender, EventArgs e)
        {
            //向Manager中注入一个变量，可以在引擎执行代码中调用，变量名为tbxVarName中配置的
            //这个功能可以注入一个服务器端存储或读取数量的类，方便交互,
            //在引擎代码中可以直接调用test.MessageBox("内容")来与主程序交互功能
            _manager?.Injection(tbxVarName.Text, new InjectionTest());
            appendMsg($"注入了变量名为{tbxVarName.Text}，可在引擎代码中执行{tbxVarName.Text}.{nameof(InjectionTest.MessageBox)}(\"消息\");或{tbxVarName.Text}.{nameof(InjectionTest.GetValue)}(\"内容\")");
        }
        #endregion

        private void btnTestBaidu_Click_1(object sender, EventArgs e)
        {
            if(_manager==null) return;
            _manager.Maximize();//最大化网页
            var baidu = new WebPage(_manager);
            baidu.Url = "https://www.baidu.com";
            #region 第一步，在输入框内输入内容
            var inputAction = new InputAction();
            inputAction.Name = "输入查询内容";
            inputAction.InputValue = "最近天气";
            inputAction.ElementFinder = new ElementFinder("kw", "id");
            baidu.Actions.Add(inputAction);
            #endregion

            #region 第二步点击百度一下查询按键
            var clickAction=new ClickAction();
            clickAction.Name = "百度一下";
            clickAction.ElementFinder = new ElementFinder("su","id");
            baidu.Actions.Add(clickAction);
            #endregion

            #region 第三步从界面上取一些值
            var outputAction=new OutputAction();
            outputAction.Name = "取当前城市";
            //使用XPath
            outputAction.ElementFinder = new ElementFinder("/html/body/div[3]/div[3]/div[1]/div[3]/div[1]/div/div/div[1]/div/div[1]/div[2]/div[1]/div[1]/div/div[1]");
            outputAction.Attributes.Add("innerText", "所在城市");
            baidu.Actions.Add(outputAction);
            #endregion

            #region 第四步从界面上取天气
            var outputAction2 = new OutputAction();
            outputAction2.Name = "取天气";
            //使用XPath
            outputAction2.ElementFinder = new ElementFinder(".weather-main-temp_6CEpK > span:nth-child(1)","css");
            outputAction2.Attributes.Add("innerText", "天气");
            baidu.Actions.Add(outputAction2);
            #endregion

            #region 第五步：测试下本地Action
            var jsAction = new MyJsAction();
            jsAction.Name = "测试JS返回";
            //使用XPath
            jsAction.Js = "return '测试JS返回';";
            baidu.Actions.Add(jsAction);
            #endregion
            var result =baidu.Execute();
            appendMsg($"执行结果:{Environment.NewLine}{result}");

        }
    }
    #region 注入的对象
    internal class InjectionTest
    {
        public void MessageBox(string msg)
        {
            System.Windows.Forms.MessageBox.Show(msg);
        }
        public string GetValue(string key)
        {
            return $"Key={key}";
        }
    }
    #endregion
}
